#-*- coding:utf-8 -*-
import re
import operator
import os
import model, util
import itertools
from collections import Counter
import xml.etree.cElementTree as ET

class Extract:
    def __init__(self, filename):
        self.filename = filename
        self.nodes = {}#Alle Knoten werden hier gespeichert
        self.ways = []#Alle "Striche" - am wichtigsten die straßen, aber auch gebäude
        self.intersections = {} #kreuzungen
        self.landuse = []
        self.trafficsignals = []#Ampeln
        self.crossroads = []
        self.x_nodes = [] #Liste mit Knoten nach x position sortiert
        self.version = "V0.6.6.0" #Release,Version,major(änderung in der ausgabe),minor
        self.minmax = {} # min und max values für die koorinaten

    def run(self):
        self.read_file()
        self.sort_ways()
        #self.fuse_ways()# Könte Probleme bei Kreuzungen geben, wenn wege zusammegefügt werden, darum ersteinmal deaktivieren
        self.clean_nodes()
        self.sort_nodes()
        self.run_inside_spawner()#vor clean_intersections
        self.clean_intersection()
        self.add_intersections_to_ways()#fürgt referenzen auf intersections in die wege ein
        self.run_waydistances()
        self.make_roundabouts()#Kreisverkehre erstellen
        self.split_ways()
        self.make_crossroads()#Kreuzungen mit ampeln machen
        self.run_waydistances()
        self.make_rights_map()#vorfahrtsregelungen erstellen
        self.get_min_max_nodes()
        self.simplify_way_ids()
        self.simplify_node_ids()
        self.remove_doublespawmer()

        self.write()

        return (self.nodes, self.ways, self.intersections, \
                self.landuse, self.crossroads)

    def read_file(self):
        datei = open("input/" + self.filename + ".xml")
        ignore_end = False #Ignoriert den rest der tags

        flag_way = None#Flag, ob gerade ein weg gelesen wird bzw. die das Objekt des aktuellen Weges

        for line in datei:
            if "<nd" in line and flag_way != None:#wenn referenz auf knoten und momentan weg betrachtet
                ref = int(re.findall(r"ref=\"(\d*?)\"", line)[0])
                flag_way.add_node(self.nodes[ref])#fügt knotenreferenz zum aktuellen weg hinzu

                try:
                    tmp_inj[ref].add_way(flag_way)
                except KeyError:
                    tmp_inj[ref] = model.intersection(ref)
                    tmp_inj[ref].add_way(flag_way)

            elif flag_way != None and "<tag k=\"lanes\"" in line:
                try:
                    flag_way.lanes = int(re.findall(r"v=\"(\d*?)\"",line)[0])
                except IndexError:
                    flag_way.lanes = 2


            elif flag_way != None and "<tag k=\"maxspeed\"" in line:
                try:
                    flag_way.speed = int(re.findall(r"v=\"(\d*?)\"",line)[0])
                except IndexError:
                    pass
                    #Falsch getaggte Maxgeschwindigkeit
            elif flag_way != None and "<tag k=\"oneway\"" in line:
                if "v=\"yes\"" in line:
                    flag_way.oneway = "Forward"
                elif "v=\"-1\"" in line:
                    flag_way.oneway = "Backward"

            elif "<node" in line:#data für einen knoten
                m = re.findall(r"id=\"(\d*?)\".lat=\"(\S*?)\".lon=\"(\S*?)\"", line)[0]
                akt_node =  model.node(int(m[0]), float(m[2]), float(m[1]))
                self.nodes[int(m[0])] = akt_node

            elif "<tag k=\"highway\" v=\"traffic_signals" in line:
                n_inter = model.intersection(akt_node.id)
                n_inter.trafficsignal = [True]
                self.intersections[akt_node.id] = n_inter
                self.trafficsignals.append(n_inter)

            elif flag_way != None and "<tag k=\"landuse" in line: #wenn weg betrachtet, und es ist eine Benutzungsfläche
                try:
                    art = re.findall(r"v=\"(.*?)\"",line)[0]
                    if art in ['commercial', 'industrial',\
                               'residential', 'retail']:
                        self.landuse.append(model.spawner(int(flag_way), \
                                            art, flag_way.nodes))

                except IndexError:
                    print line

            elif flag_way != None and "<tag k=\"highway" in line: #wenn weg betrachtet, und es ist eine straße

                art = re.findall(r"v=\"(\S*?)\"",line)[0]#v=art/größe der straße"

                flag_way.type = art

                #kreuzungen für falsche straßentypen löschen
                if art in ['motorway', 'motorway_link', 'primary',\
                           'living_street', 'unclassified','residential',\
                           'tertiary', 'secondary']:

                    self.ways.append(flag_way)

                    #alle temporären Kreuzungen die an staßen liegen übertragen
                    for n_id in tmp_inj:
                        try:
                            self.intersections[n_id].add_ways(tmp_inj[n_id].get_ways())
                        except KeyError:
                            self.intersections[n_id] = tmp_inj[n_id]
                else:
                    ignore_end = True


            elif flag_way != None and "<tag k=\"name" in line and not ignore_end: #wenn weg betrachtet, und es ist eine straße, suche nach name
                name = re.findall(r"v=\"(.*?)\"",line)[0]#name der straße"

                flag_way.name = re.sub(r"&#39;","'",name)


            elif "<way" in line:#begin einer weg sektion
                flag_way = model.way(int(re.findall(r"id=\"(\d*?)\"",line)[0]))
                tmp_inj = {}
                ignore_end = False

            elif "</way" in line:#ende einer weg sektion
                flag_way = None
                ignore_end = False


        datei.close()


    #sortieren der wege
    def sort_ways(self):
        self.ways = sorted(self.ways)

    def simplify_way_ids(self):
        self.ways = filter(lambda x: len(x.intersections) != 0, self.ways)
        map(lambda x: util.set_way_id(x),self.ways)

    def simplify_node_ids(self):
        util.NODE_LIST = self.nodes
        i = 0
        for node in self.nodes.values():
            node.shortid = i
            i += 1


    def fuse_ways(self):
        ways_zus = []
        #zusammenfügen "zerstüclkelter" Wege
        last_way = model.way(000) #Dummy
        tmp_list= []
        for way in self.ways:

            if way.hasNoName(): #damit er nicht alle Namenlosen versuch zu verbinden
                ways_zus.append(way)
            elif way.name == last_way.name: #gleicher weg wie zuvor
                tmp_list.append(way)
            else:
                #in tmp_list ist jetzt eine liste von gleichen wegen
                anz = len(tmp_list) + 1
                while anz > len(tmp_list):
                    anz = len(tmp_list)

                    for weg1 in tmp_list:
                        for weg2 in tmp_list:
                            #Wenn Gleicher name + Gleicher end/startpunkt, dann zusammenfügen
                            if weg1.fuseable(weg2):
                                weg1.add_nodes(weg2.nodes[1:])
                                tmp_list.remove(weg2)

                #Unveränderterte straßen + zusammengefügte straßen

                ways_zus.extend(tmp_list)

                tmp_list = [way]
                last_way = way

        self.ways = ways_zus

    def clean_nodes(self):
        """
        Entfernt alle Knoten die nicht mehr benutzt werden

        Muss nach clean intersections durchgeführt werden
        """
        nodes_new = {}
        for way in self.ways:
            for node in way.nodes:
                nodes_new[node.id] = node

        for use in self.landuse:
            for node in use.nodes:
                nodes_new[node.id] = node

        #Theoretisch unnötig, aber man weiß ja nie
        for inter in self.intersections.values():
            nodes_new[inter.node] =self.nodes[inter.node]

        self.nodes = nodes_new

    def clean_intersection(self):
        """
        Entfernt alle "Kreuzungen" Die entweder nur an einer straße liegen,
        oder keine Ampel haben (Das heißt nur wegpunkte ohne besonderheiten sind)
        """
        intersections = {}
        for inter_id in self.intersections:
            if len(self.intersections[inter_id]) > 1 \
            or self.intersections[inter_id].spawner \
            or len(self.intersections[inter_id].trafficsignal) > 0: #alle knoten entfernen, die nur eine straße enthalten (= keine Kreuzung)(+Ampeln)
                intersections[inter_id] = self.intersections[inter_id]
        self.intersections = intersections

    def run_waydistances(self):
        for way in self.ways:
            way.make_distances()

    def remove_doublespawmer(self):
        used_nodes = []
        for spwaner in self.landuse:
            for node in spwaner.spawnpoints:
                if node in used_nodes:
                    #print str(node) +  " Entfernt"
                    spwaner.spawnpoints.remove(node)
                else:
                    used_nodes.append(node)
            if spwaner.spawnpoints == []:
                self.landuse.remove(spwaner)


    def make_rights_map(self):
        map(lambda x:x.make_right(self.nodes, self.ways), self.intersections.values())
        #[x.make_right(self.nodes) for x in self.intersections]

    def add_intersections_to_ways(self):
        """
        for intersection in self.intersections.values():
            for way in intersection.ways:
                way.intersections.append(intersection)
        map(lambda way: way.intersections.sort(key = lambda x: way.nodes.index(self.nodes[x.node])),self.ways)
        """
        map(lambda way: way.intersections.extend(map(lambda n: self.intersections[n.id],filter(lambda node: node.id in self.intersections.keys() ,way.nodes)))  , self.ways)


    def get_min_max_nodes(self):
        """
        Hohlt sich die maximalen coordinaten die eine Node annimmt
        """

        self.minmax["max_x"]  = max(self.nodes.values(), key=lambda n: n.x).x
        self.minmax["min_x"]  = min(self.nodes.values(), key=lambda n: n.x).x
        self.minmax["max_y"]  = max(self.nodes.values(), key=lambda n: n.y).y
        self.minmax["min_y"]  = min(self.nodes.values(), key=lambda n: n.y).y
        #print self.minmax

    def split_ways(self):
        """
        Spaltet die Wege an den kreuzungen auf (macht zum beispiel Abbiegen einfacher)
        """
        map(lambda x:x.split_all(self.nodes, self.ways), self.intersections.values())

    def make_crossroads(self):
        crossroads = set()
        for item in self.trafficsignals:
            node = self.nodes[item.node]

            pointxup = node.x + 0.00069
            pointxdown = node.x - 0.00069
            pointyup = node.y + 0.00069
            pointydown = node.y - 0.00069



            #possibles = filter(lambda inter:self.nodes[inter.node].x < pointxup \
            #                                and self.nodes[inter.node].x > pointxdown \
            #                                and self.nodes[inter.node].y < pointyup \
            #                                and self.nodes[inter.node].y > pointydown , self.trafficsignals)

            possibles = []
            for inter in self.trafficsignals:
                if  self.nodes[inter.node].x > pointxdown \
                                            and self.nodes[inter.node].x < pointxup \
                                            and self.nodes[inter.node].y < pointyup \
                                            and self.nodes[inter.node].y > pointydown:

                    possibles.append(inter)

            crossroads.add(model.crossroad(possibles))


        newcrossroads = set(list(crossroads))
        #print len(newcrossroads)
        for cross1 in crossroads:
            set1 = frozenset(cross1.signals)
            for cross2 in crossroads:
                set2 = frozenset(cross2.signals)
                if set2 < set1:
                    newcrossroads.discard(cross2)
        #print len(newcrossroads)
        """
        inters = []
        map(lambda x:inters.extend(x.signals),newcrossroads)

        groups = [list(g) for k, g in itertools.groupby(inters)]
        print groups
        """
        inters = {} #intersection:crossroad
        for cross in newcrossroads:
            for signal in cross.signals:
                if signal in inters:
                    #doppelt erfasste ampel (teste nun abstände zu den beiden clustern)
                    cross1 = cross
                    cross2 = inters[signal]
                    cross1_distances = []
                    cross2_distances = []

                    for vergl_kn in cross1.signals:
                        dist = util.quick_node_distance(self.nodes[vergl_kn.node], self.nodes[signal.node])
                        if dist != 0:
                            cross1_distances.append(dist)

                    for vergl_kn in cross2.signals:
                        dist = util.quick_node_distance(self.nodes[vergl_kn.node], self.nodes[signal.node])
                        if dist != 0:
                            cross2_distances.append(dist)
                    if cross1_distances == [] : cross1_distances = [10]#entfernung hoch setzten, wenn nur eine Ampel im cluster
                    if cross2_distances == [] : cross2_distances = [10]
                    if min(cross1_distances) < min(cross2_distances):
                        #Wenn der abstand zu eienm Knoten der ersten crossroad kleiner ist
                        cross2.signals.remove(signal)
                        inters[signal] = cross1
                    else:
                        cross1.signals.remove(signal)

                else:
                    inters[signal] = cross



        for cross in newcrossroads:
            cross.order(self.nodes)
            self.crossroads.append(cross)
        #print len(self.crossroads)

        nc = []
        for cross in self.crossroads:
            cross_inner_streetnode = []#liste mit intersections die in der crossroad vorhanden sind
            for inter in cross.signals:
                if len(cross.signals) == 1:
                    if len(inter.ways) == 2:
                        #zwei straßen an der intersection, und nur eine ampel, gilt also für beide
                        inter.trafficsignal = inter.ways
                    else:# mehr oder weniger straßen an einer einzelenen ampeln sind ungut -> entfernen
                        #print str(inter.node) + "-" +str(len(inter.ways))
                        inter.trafficsignal = []
                        self.trafficsignals.remove(inter)
                        cross.signals.remove(inter)

                else:
                    if len(inter.ways) > 1 and (inter.ways[0].oneway == "Forward" and inter.ways[0].intersections[-1] == inter or \
                        inter.ways[0].oneway =="Backward" and inter.ways[0].intersections[0] == inter):
                        #Einbahnstraße in richtung Ampel
                        inter.trafficsignal = [inter.ways[0]]
                    elif len(inter.ways) > 1 and (inter.ways[1].oneway == "Forward" and inter.ways[1].intersections[-1] == inter or \
                        inter.ways[1].oneway =="Backward" and inter.ways[1].intersections[0] == inter):
                        #Einbahnstraße in richtung Ampel
                        inter.trafficsignal = [inter.ways[1]]
                    else:
                        #keine Einbahnstraße
                        """
                        alle intersections einer crossroad durchgehen,
                        dabei alle nodes sammeln, das max ist der node in der mitte,
                        alle straßen mit dem node in der mitte am anfang oder ende bei den relevandten straßen rausschmeißen

                        """
                        inter.trafficsignal = inter.ways #dummy
                        #Achstung evtl folgende zeilen weiter einrücken /wenigerweit
                        for way in inter.ways:#für jede straße die erste und letzte intersection hinzufügen
                            cross_inner_streetnode.append(way.intersections[0])
                            cross_inner_streetnode.append(way.intersections[-1])

            if cross_inner_streetnode != []: #wenn nodes eingelegt wurden
                #maxnode (innersternode)
                center = Counter(cross_inner_streetnode).most_common(1)[0][0]
                for inter in cross.signals:
                    for way in inter.trafficsignal:
                        if way.intersections[0] == center or way.intersections[-1] == center:
                            #es ist eine Innere straße, da sie an dem zentralen knoten liegt
                            inter.trafficsignal.remove(way)
                #eine zweiten inneren knoten?
                #print Counter(cross_inner_streetnode).most_common(2)
                if Counter(cross_inner_streetnode).most_common(2)[1][1] == Counter(cross_inner_streetnode).most_common(2)[0][1]:
                    #ja
                    #print "ai"
                    center = Counter(cross_inner_streetnode).most_common(2)[1][0]
                    for inter in cross.signals:
                        for way in inter.trafficsignal:
                            if way.intersections[0] == center or way.intersections[-1] == center:
                                #es ist eine Innere straße, da sie an dem zentralen knoten liegt
                                inter.trafficsignal.remove(way)




            if cross.signals != []:
                nc.append(cross)

        self.crossroads = nc




        """

        clusters = self.cluster_trafficsignals_fast()
        for inter_list in clusters:
            clust = model.crossroad(inter_list)
            clust.order(self.nodes)
            self.crossroads.append(clust)
        """

    def cluster_trafficsignals(self):
        return cluster.HierarchicalClustering(self.trafficsignals, lambda x,y: util.quick_node_distance(self.nodes[x.node],self.nodes[y.node])).getlevel(0.0005)

    def run_inside_spawner(self):
        for spawner in self.landuse:
            spawner.get_points(self.x_nodes, self.intersections)

        self.landuse = filter(lambda x: len(x.spawnpoints) > 0, self.landuse) #spawner ohne node rausfiltern


    def sort_nodes(self):
        self.x_nodes = sorted(self.nodes.values(),key= lambda x:x.x)

    def make_roundabouts(self):
        map(lambda x: x.checkroundabout(), self.ways)

    def write(self):

        #Momentan alle knoten, lässt sich ggf. optimieren indem nach relevandten Konoten gefiltert wird
        #nodeid;coodrinate1:coord2
        baseDir = "output/" + self.filename + "/"
        if(not(os.path.isdir(baseDir))):
            os.mkdir(baseDir)
        datei_nodes = open(baseDir + "nodes_"+ self.filename +"_tmp.dat", "w")
        datei_nodes.write(self.version + "\n")
        datei_nodes.write(str(len(self.nodes.values())) + "\n")
        datei_nodes.write(str(self.minmax["min_x"]) + "," + str(self.minmax["min_y"])\
        + "," +  str(self.minmax["max_x"]) + "," + str(self.minmax["max_y"]) + "\n")
        for node in self.nodes.values():
            datei_nodes.write(str(node) + "\n")
        datei_nodes.close()


        #nodeid;str_w_trafficsignal1:str_w_trafficsignal1...;spawner;wayid1:wayid2:wayid3...
        datei_intersection = open(baseDir + "/intersection_"+ self.filename +"_tmp.dat", "w")
        datei_intersection.write(self.version + "\n")
        for inter in self.intersections.values():
            datei_intersection.write(str(inter) + "\n")
        datei_intersection.close()

        #wayid;Straßenname;type;oneway;speed;lines;länge;nodeid1:nodeid2:...:nodeidn
        datei_ways = open(baseDir + "ways_"+ self.filename +"_tmp.dat", "w")
        datei_ways.write(self.version + "\n")
        datei_ways.write(str(len(self.ways)) + "\n")
        for way in self.ways:
                datei_ways.write(str(way) + "\n")
        datei_ways.close()


        #gekürzte straßenausgabe in xml
        root = ET.Element("root")
        for way in self.ways:
            way.get_et(root)
        tree = ET.ElementTree(root)
        tree.write(baseDir + "fullways_"+ self.filename +"_tmp.xml")


        # sp_id;typ;größe;inter_id,strid_1,strid_2,strid_n:inter_id2,strid_m
        datei_landuse = open(baseDir + "spawns_"+ self.filename +"_tmp.dat", "w")
        datei_landuse.write(self.version + "\n")
        for spawner in self.landuse:
            datei_landuse.write(spawner.string(self.intersections) + "\n")
        datei_landuse.close()

        #intersectionsids_an_denen_ampeln_sind:Inter....
        datei_crossroads = open(baseDir + "crossroads_"+ self.filename +"_tmp.dat", "w")
        datei_crossroads.write(self.version + "\n")
        for cr in self.crossroads:
            datei_crossroads.write(str(cr) + "\n")
        datei_crossroads.close()
