#-*- coding:utf-8 -*-
"""
Tools für das Preprocessing
"""
import math
import util

util.split_counter = 1
util.WAY_ID = 0
util.NODE_LIST = None


def set_way_id(way):
    """
    Setzt eine neue id für einen Weg
    """
    way.id = util.WAY_ID
    util.WAY_ID += 1


def higher_rank(way1, way2):
    """
    Prüft ob way1 einen höheren rank als way2 hat
    """
    rank = ["roundabout",  "motorway", "motorway_link", "primary", \
            "secondary", "tertiary", "residential", "living_street", \
            "unclassified"]#umso weiter vorne umsowichtiger
    pos1 = rank.index(way1.type)
    pos2 = rank.index(way2.type)
    return pos1 > pos2


def node_distance(node1, node2):
    """
    Knotendistanz in metern
    """
    return distance_on_unit_sphere(node1.x, node1.y, \
                                   node2.x, node2.y) * 6373 * 1000


def quick_node_distance(node1, node2):
    """
    Gibt die Euklidische Distanz zwischen zwei Kooridnaten aus
    """
    if node1 == node2:
        return 0
    return math.sqrt((node1.x - node2.x)**2 + (node1.y - node2.y)**2)


def distance_on_unit_sphere(lat1, long1, lat2, long2):
    """
    http://www.johndcook.com/python_longitude_latitude.html
    """

    # Convert latitude and longitude to
    # spherical coordinates in radians.
    degrees_to_radians = math.pi/180.0

    # phi = 90 - latitude
    phi1 = (90.0 - lat1)*degrees_to_radians
    phi2 = (90.0 - lat2)*degrees_to_radians

    # theta = longitude
    theta1 = long1*degrees_to_radians
    theta2 = long2*degrees_to_radians

    # Compute spherical distance from spherical coordinates.

    # For two locations in spherical coordinates
    # (1, theta, phi) and (1, theta, phi)
    # cosine( arc length ) =
    #    sin phi sin phi' cos(theta-theta') + cos phi cos phi'
    # distance = rho * arc length

    cos = (math.sin(phi1)*math.sin(phi2)*math.cos(theta1 - theta2) +
           math.cos(phi1)*math.cos(phi2))
    arc = math.acos( cos )

    # Remember to multiply arc by the radius of the earth
    # in your favorite set of units to get length.
    return arc


def schnittwinkel(node1, node2, node3, node4):
    """
    Gerade zwischen 1 und 2 wird mit gerade zwischen 3 uund 4 geschnitten
    """
    if node1 == node2 or node3 == node4:
        return 0

    vec1x = node1.x - node2.x
    vec1y = node1.y - node2.y

    vec2x = node3.x - node4.x
    vec2y = node3.y - node4.y
    winkel = math.degrees(math.acos((vec1x * vec2x + vec1y * vec2y)/ \
                         (math.sqrt(vec1x**2 + vec1y**2) * \
                          math.sqrt(vec2x**2 + vec2y**2)) % 1))

    return winkel


def schnittwinkel_360(node1, node2, node3, node4):
    """
    Gerade zwischen 1 und 2 wird mit gerade zwischen 3 uund 4 geschnitten
    """
    if node1 == node2 or node3 == node4:
        return 0

    vec1x = node1.x - node2.x
    vec1y = node1.y - node2.y

    vec2x = node3.x - node4.x
    vec2y = node3.y - node4.y

    a1 = math.atan2(vec1x, vec1y)
    a2 = math.atan2(vec2x, vec2y)
    rdeg = a1 - a2
    if rdeg < 0:
        rdeg += 2*math.pi
    return math.degrees(rdeg)


def schnittwinkel_alt(node1, node2, node3, node4):
    """
    Gerade zwischen 1 und 2 wird mit gerade zwischen 3 uund 4 geschnitten
    """


    vec1x = node1.x - node2.x
    vec1y = node1.y - node2.y
    vec2x = node3.x - node4.x
    vec2y = node3.y - node4.y

    winkel = math.degrees(math.acos(abs((vec1x * vec2x + vec1y * vec2y)\
                        / (math.sqrt(vec1x**2 + vec1y**2) \
                           * math.sqrt(vec2x**2 + vec2y**2)))))

    return winkel


def point_inside_polygon(x, y, poly):
    """
    http://www.ariel.com.au/a/python-point-int-poly.html
    """
    n = len(poly)
    inside = False

    p1x, p1y = poly[0]
    for i in range(n + 1):
        p2x, p2y = poly[i % n]
        if y > min(p1y, p2y):
            if y <= max(p1y, p2y):
                if x <= max(p1x, p2x):
                    if p1y != p2y:
                        xinters = (y-p1y)*(p2x - p1x)/(p2y - p1y) + p1x
                    if p1x == p2x or x <= xinters:
                        inside = not inside
        p1x, p1y = p2x, p2y

    return inside

def poly_area2D(poly):
    """
    http://code.activestate.com/recipes/578275-2d-polygon-area/
    """
    total = 0.0
    n = len(poly)
    for i in range(n):
        v1 = poly[i]
        v2 = poly[(i+1) % n]
        total += v1[0] * v2[1] - v1[1] * v2[0]
    return abs(total/2)
