/*
	Serial traffic simulation program for PaPo.

	############################################

	This application uses data from Open Street Map, henceforth
	that data is made available under the Open Database License: http://opendatacommons.org/licenses/odbl/1.0/.
	Any rights in individual contents ofthe database are licensed under 
	the Database Contents License: http://opendatacommons.org/licenses/dbcl/1.0/
	See more at: http://opendatacommons.org/licenses/odbl/#sthash.y0L9Yy1N.dpuf

	############################################

	Author: Martin Poppinga, Miriam von Heeremann, Marcel Ellermann
	Date: 24.07.2013      
	Distributed under the Boost Software License, Version 1.0.
	    See accompanying file LICENSE_1_0.txt or copy at
	          http://www.boost.org/LICENSE_1_0.txt)

	#############################################
*/


#pragma once


#include "WorldObj.h"
#include "NodeHandler.h"
#include "worldDefs.h"
#include <unordered_set>
#include <set>
#include <list>

#define KM_H_TO_M_S 3.6
#define KM_H_TO_L_T (KM_H_TO_M_S * SEC_TO_LOGIC_TIME)
//physical constants related to seconds
#define ACCELERATION_PER_SEC  3
#define N_ACCELERATION_PER_SEC -2
#define EM_N_ACCELERATION_PER_SEC -7
//physical constants related to internal logic time, acceleration is related to time as 
#define ACCELERATION ACCELERATION_PER_SEC / SEC_TO_LOGIC_TIME / SEC_TO_LOGIC_TIME
#define N_ACCELERATION N_ACCELERATION_PER_SEC / SEC_TO_LOGIC_TIME / SEC_TO_LOGIC_TIME
#define EM_N_ACCELERATION EM_N_ACCELERATION_PER_SEC / SEC_TO_LOGIC_TIME / SEC_TO_LOGIC_TIME

#define PRIORITY_STREET_PASS_SPEED 20 / KM_H_TO_L_T
#define NODE_LOCK_TIME_LT_F 7 * SEC_TO_LOGIC_TIME //7 seconds at max speed forward lock
#define NODE_LOCK_DISTANCE_EM_B -CAR_LENGTH*2 //car length backward lock


namespace traffic_simulation
{

	class Street;

	class Car : WorldObj
	{
	public:
		Car(ui64 guid, Node* n, Street* s, std::list<std::pair<Node*, Street*>*>* path);
		~Car();
		
		double getMetricPos() const{return _cMePos;};
		ui64 getGUID() const{return _guid;};
		Street* getCurrentStreet() const {return _cStreet;};
		Node* getCurrentGoalNode() const {return _cgNode.first;};
		Street* getCurrentGoalStreet() const {return _cgNode.second;};
		driveDirection getDirection() const {return _cDirection;};
		double getCurrentSpeed() const {return _speed;};
		void setPath(std::list<std::pair<Node*, Street*>*>* path);
		const std::list<std::pair<Node*, Street*>*>* getPath(){return _cPath;};
		void update(ui8 step = 1);
		ui32 getCurrentTimeStamp() const {return _cTimeStamp;};
		bool isPriorityLocked(ui32 cStreet, ui32 gStreet, ui32 gNode, std::set<Street*> &closedSet) const;
		void setMePosition(double pos){_cMePos = pos;};
		void setSpeed(double speed){_speed = speed;};
	private:
		double regulateSpeed();
		std::list<std::pair<Node*,Street*>> getReqStreetsForDist(double distance, bool *reachEnd);
		void checkForPriorityStreet();
		void performMovement(double acc);
		void perfomNodeLocks(bool freeAll = false);
		Street* _cStreet;
		driveDirection _cDirection;
		std::pair<Node*,Street*> _cgNode;
		bool _wPriority; //flag if we're currently waiting for a priority street
		double _speed;
		double _cMePos;
		const Car* _fCar;
		bool _notified;
		ui32 _cTimeStamp;
		std::list<ui32> _locks; //locks on nodes
		ui64 _guid;
		std::list<std::pair<Node*, Street*>*>* _cPath;
	};

}