/*
	Serial traffic simulation program for PaPo.

	############################################

	This application uses data from Open Street Map, henceforth
	that data is made available under the Open Database License: http://opendatacommons.org/licenses/odbl/1.0/.
	Any rights in individual contents ofthe database are licensed under 
	the Database Contents License: http://opendatacommons.org/licenses/dbcl/1.0/
	See more at: http://opendatacommons.org/licenses/odbl/#sthash.y0L9Yy1N.dpuf

	############################################

	Author: Martin Poppinga, Miriam von Heeremann, Marcel Ellermann
	Date: 24.07.2013      
	Distributed under the Boost Software License, Version 1.0.
	    See accompanying file LICENSE_1_0.txt or copy at
	          http://www.boost.org/LICENSE_1_0.txt)

	#############################################
*/

#pragma once
#include "boost/cstdint.hpp"
#include "Car.h"
#include <list>
#include "worldDefs.h"
#include <map>
#include <omp.h>

namespace traffic_simulation
{
	class CarHandler
	{
	public:
		CarHandler();
		~CarHandler();

		void update(uint8_t step = 1);
		Car* addCarToWorld(ui64 guid, Node* n, Street* s, std::list<std::pair<Node*, Street*>*>* path);
		void removeCarFromWorld(Car* c);
		void setCarLock(ui64 id);
		void unsetCarLock(ui64 id);
		void dumpCars();
		ui32 carCount();
		void updateCarCount();

	private:
		std::list<Car*> _lCars;
		ui32 _globalCarCount;
#ifdef CAR_DUMP
		std::ofstream* _carDumpFile;
#endif

	};
}