/*
Serial traffic simulation program for PaPo.

############################################

This application uses data from Open Street Map, henceforth
that data is made available under the Open Database License: http://opendatacommons.org/licenses/odbl/1.0/.
Any rights in individual contents ofthe database are licensed under 
the Database Contents License: http://opendatacommons.org/licenses/dbcl/1.0/
See more at: http://opendatacommons.org/licenses/odbl/#sthash.y0L9Yy1N.dpuf

############################################

Author: Martin Poppinga, Miriam von Heeremann, Marcel Ellermann
Date: 24.07.2013      
Distributed under the Boost Software License, Version 1.0.
See accompanying file LICENSE_1_0.txt or copy at
http://www.boost.org/LICENSE_1_0.txt)

#############################################
*/
#pragma once
#include "boost/cstdint.hpp"
#include "boost/lexical_cast.hpp"
#include "worldDefs.h"
#include <fstream>

#ifdef _WIN32
#define CAR_LOG_FILE "LOG\\LOG_CAR.txt"
#define TRAFFIC_LOG_FILE "LOG\\LOG_TRAFFIC.txt"
#define STREET_LOG_FILE "LOG\\LOG_STREET.txt"
#define CROSSING_LOG_FILE "LOG\\LOG_CROSSING.txt"
#define WORLD_LOG_FILE "LOG\\LOG_WORLD.txt"
#else
#define CAR_LOG_FILE "./LOG/LOG_CAR.txt"
#define TRAFFIC_LOG_FILE "./LOG/LOG_TRAFFIC.txt"
#define STREET_LOG_FILE "./LOG/LOG_STREET.txt"
#define CROSSING_LOG_FILE "./LOG/LOG_CROSSING.txt"
#define WORLD_LOG_FILE "./LOG/LOG_WORLD.txt"
#endif


namespace traffic_simulation
{
	class Log //define log as singleton
	{
	public:
		void writeStreetLog(const char* logText);
		void writeTrafficLog(const char* logText);
		void writeCrossingLog(const char* logText);
		void writeCarLog(const char* logText);
		void writeWorldLog(const char* logText);

		static Log* getInstance();
		~Log();

	private:
		Log();
		static bool _bInstance;
		static Log* _instance;
		void writeTimeStamp(std::ofstream* s, boost::uint32_t timeStamp);
		void writeLogOut(std::ofstream* s, const char* logText, boost::uint32_t timeStamp);

		std::ofstream* _streetLog;
		std::ofstream* _carLog;
		std::ofstream* _crossingLog;
		std::ofstream* _worldLog;
		std::ofstream* _trafficLog;
	};
}

