#include <mpi.h>
#include <omp.h>
#include <iostream>
#include <list>
#include <map>


#pragma once

class MessageHandler;
class MPIRcvMessage_Finalize;


struct MPIMsgContainer
{
	void* val;
	MPI_Datatype type;
	MPI_Request request;
};

class IMPIRcvMessage
{
public:
	IMPIRcvMessage(int msgID){_msgID = msgID; registerSelf();};
	virtual void handleMessage() = 0;
	virtual void* getValuePtr() = 0;
	virtual int getValueSize() = 0;
	virtual int getValueCount(){return 1;};
	virtual MPI_Datatype getDataType(){return MPI_INTEGER;};
	virtual MPI_Comm getComm(){return MPI_COMM_WORLD;};
	virtual int getSource(){return MPI_ANY_SOURCE;};
protected:
	void registerSelf();
	int _msgID;
};



class MessageHandler
{
public:
	friend MPIRcvMessage_Finalize;
	friend IMPIRcvMessage;
	static void Init(int mpiThreadSupport);
	static MessageHandler* getInstance();
	/* Sending Methods */
	//Wait till all pending Messages have been send
	void WaitMsgsSend();
	void Synchronize();
	void WaitTillReadyForRcv(){while(!_readyForRcv){};};
	void SendMessage(void* val,int count, int msgCode, int rcvr);
	void FinalizeHandler();
	static bool IsRunning() {return _instance != NULL;};
	void RunRcvManager();
private:
	int _rank, _commSize, _mpiThreadSupport;
	bool _isRunning;
	IMPIRcvMessage* getMessageHandler(int msgID);
	bool _finalizeHandler;
	bool _readyForRcv;
	bool _stopAcceptMessage;
	bool _isUpdating;
	bool _receiveMessages;
	void _message_rcv_handler();
	std::map<MPI_Request,MPIMsgContainer*> _msgs;
	std::map<int,IMPIRcvMessage*> _msgHandler;
	static MessageHandler* _instance;
};
