/*
Serial traffic simulation program for PaPo.

############################################

This application uses data from Open Street Map, henceforth
that data is made available under the Open Database License: http://opendatacommons.org/licenses/odbl/1.0/.
Any rights in individual contents ofthe database are licensed under 
the Database Contents License: http://opendatacommons.org/licenses/dbcl/1.0/
See more at: http://opendatacommons.org/licenses/odbl/#sthash.y0L9Yy1N.dpuf

############################################

Author: Martin Poppinga, Miriam von Heeremann, Marcel Ellermann
Date: 24.07.2013      
Distributed under the Boost Software License, Version 1.0.
See accompanying file LICENSE_1_0.txt or copy at
http://www.boost.org/LICENSE_1_0.txt)

#############################################
*/

#include "Messages.h"
#include "Log.h"
#include "boost/format.hpp"
#include "worldDefs.h"
#include "MapHandler.h"
#include "Street.h"
#include <list>

using namespace traffic_simulation;

void MPIRcvMessage_Car::handleMessage()
{
	std::list<std::pair<Node*,Street*>*>* _path = new std::list<std::pair<Node*,Street*>*>();
	Street* s = MAP->getStreet(value._cStreet);
	for(int i = 0; i < value.pathSize; i++)
	{
		_path->push_back(new std::pair<Node*, Street*>(NODES->getNode(value._path[i][0]),MAP->getStreet(value._path[i][1])));
	}
	Node* insertNode = (value._path[0][0] == (*s->nodes()->begin())->getID()) ? (*s->nodes()->rbegin()): (*s->nodes()->begin());
	Car* c = MAP->getCarHandler()->addCarToWorld(value._guid, insertNode, s, _path);
	c->setMePosition(value._cMePosition);
	c->setSpeed(value._cSpeed);
}

void MPIRcvMessage_CtoB::handleMessage()
{
	MAP->getStreet(value._streetID)->notifyCloseToBegin(value._guid,value._streetID);
}

void MPIRcvMessage_NCtoB::handleMessage()
{
	MAP->getStreet(value._streetID)->notifyNotCloseToBegin(value._guid);
}

void MPIRcvMessage_CtoE::handleMessage()
{
	MAP->getStreet(value._CstreetID)->notifyCloseToEnd(value._guid,value._GnodeID,value._GstreetID);
}

void MPIRcvMessage_NCtoE::handleMessage()
{
	MAP->getStreet(value._CstreetID)->notifyNotCloseToEnd(value._guid);
}

MPIRcvMessage_NCtoE::MPIRcvMessage_NCtoE() : IMPIRcvMessage(MSG_NOT_CLOSE_TO_END)
{
	int b_Length[2]={1,1};
	MPI_Aint t_disp[2]={0,8};
	MPI_Datatype m_types[2]={MPI_UNSIGNED_LONG, MPI_UNSIGNED};
#pragma omp critical (MPI_FUNC_CALL)
	{
		MPI_Type_create_struct(2, b_Length, t_disp, m_types, &_type);
		MPI_Type_commit(&_type);
	}
}

MPIRcvMessage_CtoE::MPIRcvMessage_CtoE() : IMPIRcvMessage(MSG_CLOSE_TO_END)
{
	int b_Length[4]={1,1,1,1};
	MPI_Aint t_disp[4]={0,8,12,16};
	MPI_Datatype m_types[4]={MPI_UNSIGNED_LONG, MPI_UNSIGNED, MPI_UNSIGNED, MPI_UNSIGNED};
#pragma omp critical (MPI_FUNC_CALL)
	{
		MPI_Type_create_struct(4, b_Length, t_disp, m_types, &_type);
		MPI_Type_commit(&_type);
	}
}

MPIRcvMessage_CtoB::MPIRcvMessage_CtoB() : IMPIRcvMessage(MSG_CLOSE_TO_BEGIN)
{
	int b_Length[3]={1,1,1};
	MPI_Aint t_disp[3]={0,8,12};
	MPI_Datatype m_types[3]={MPI_UNSIGNED_LONG, MPI_UNSIGNED, MPI_UNSIGNED};
#pragma omp critical (MPI_FUNC_CALL)
	{
		MPI_Type_create_struct(3, b_Length, t_disp, m_types, &_type);
		MPI_Type_commit(&_type);
	}
}

MPIRcvMessage_NCtoB::MPIRcvMessage_NCtoB() : IMPIRcvMessage(MSG_NOT_CLOSE_TO_BEGIN)
{
	int b_Length[2]={1,1};
	MPI_Aint t_disp[2]={0,8};
	MPI_Datatype m_types[2]={MPI_UNSIGNED_LONG, MPI_UNSIGNED};
#pragma omp critical (MPI_FUNC_CALL)
	{
		MPI_Type_create_struct(2, b_Length, t_disp, m_types, &_type);
		MPI_Type_commit(&_type);
	}
}

MPIRcvMessage_Car::MPIRcvMessage_Car() : IMPIRcvMessage(MSG_CAR)
{
	int b_Length[6]={1,1400,1,1,1,1};
	MPI_Aint t_disp[6]={0,8,5608,5616,5624,5632};
	MPI_Datatype m_types[6]={MPI_UNSIGNED_LONG, MPI_UNSIGNED, MPI_UNSIGNED, MPI_DOUBLE, MPI_DOUBLE, MPI_UNSIGNED_SHORT};
#pragma omp critical (MPI_FUNC_CALL)
	{
		MPI_Type_create_struct(6, b_Length, t_disp, m_types, &_type);
		MPI_Type_commit(&_type);
	}
}
