#include "MessageHandler.h"
#include "worldDefs.h"
#include "boost/cstdint.hpp"
/*
Messages that for the Message Handler are defined within here
*/

#pragma once 

#define MSG_TEST 0
#define MSG_CAR  1
#define MSG_CLOSE_TO_END 2
#define MSG_NOT_CLOSE_TO_END 3
#define MSG_CLOSE_TO_BEGIN 4
#define MSG_NOT_CLOSE_TO_BEGIN 5
#define MSG_SYNC 6

//basic test message
//implement the IMPIRcvMessage interface
//it's important to create the messages before we start the Message receive handler
//otherwhise later messages will be ignored
class MPIRcvMessage_Test : public virtual IMPIRcvMessage
{
public:
	MPIRcvMessage_Test() : IMPIRcvMessage(MSG_TEST){
		// you can do stuff here..
		//but don't forget to call the base constructor
	};
	//requried methods
	void* getValuePtr(){return &value;};
	void handleMessage(){
#pragma omp critical(cout)
		{
			std::cout << "Test Message handled. " << std::endl;
		}};
private:
	int value;
};

struct __carStoreStruct
{
	uint64_t _guid;
	uint32_t _path[700][2];
	uint32_t _cStreet;
	double _cSpeed;
	double _cMePosition;
	uint16_t pathSize;
};

class MPIRcvMessage_Car : public virtual IMPIRcvMessage
{
public:
	MPIRcvMessage_Car();
	void* getValuePtr(){return &value;};
	int getValueSize(){return sizeof(value);};
	void handleMessage();
	MPI_Datatype getDataType(){return _type;};
private:
	__carStoreStruct value;
	MPI_Datatype _type;
};

struct __CloseToEndStoreStruct
{
	ui64 _guid;
	ui32 _CstreetID;
	ui32 _GnodeID;
	ui32 _GstreetID;
};

class MPIRcvMessage_CtoE : public virtual IMPIRcvMessage
{
public:
	MPIRcvMessage_CtoE();
	void* getValuePtr(){return &value;};
	int getValueSize(){return sizeof(value);};
	void handleMessage();
	MPI_Datatype getDataType(){return _type;};
private:
	__CloseToEndStoreStruct value;
	MPI_Datatype _type;
};
struct __NotCloseToEndStoreStruct
{
	ui64 _guid;
	ui32 _CstreetID;
};
class MPIRcvMessage_NCtoE : public virtual IMPIRcvMessage
{
public:
	MPIRcvMessage_NCtoE();
	void* getValuePtr(){return &value;};
	int getValueSize(){return sizeof(value);};
	void handleMessage();
	MPI_Datatype getDataType(){return _type;};
private:
	__NotCloseToEndStoreStruct value;
	MPI_Datatype _type;
};

struct __CloseToBeginStoreStruct
{
	ui64 _guid;
	ui32 _nodeID;
	ui32 _streetID;
};

class MPIRcvMessage_CtoB : public virtual IMPIRcvMessage
{
public:
	MPIRcvMessage_CtoB();
	void* getValuePtr(){return &value;};
	int getValueSize(){return sizeof(value);};
	void handleMessage();
	MPI_Datatype getDataType(){return _type;};
private:
	__CloseToBeginStoreStruct value;
	MPI_Datatype _type;
};

struct __NotCloseToBeginStoreStruct
{
	ui64 _guid;
	ui32 _streetID;
};

class MPIRcvMessage_NCtoB : public virtual IMPIRcvMessage
{
public:
	MPIRcvMessage_NCtoB();
	void* getValuePtr(){return &value;};
	int getValueSize(){return sizeof(value);};
	void handleMessage();
	MPI_Datatype getDataType(){return _type;};
private:
	__NotCloseToBeginStoreStruct value;
	MPI_Datatype _type;
};

