/*
Serial traffic simulation program for PaPo.

############################################

This application uses data from Open Street Map, henceforth
that data is made available under the Open Database License: http://opendatacommons.org/licenses/odbl/1.0/.
Any rights in individual contents ofthe database are licensed under 
the Database Contents License: http://opendatacommons.org/licenses/dbcl/1.0/
See more at: http://opendatacommons.org/licenses/odbl/#sthash.y0L9Yy1N.dpuf

############################################

Author: Martin Poppinga, Miriam von Heeremann, Marcel Ellermann
Date: 24.07.2013      
Distributed under the Boost Software License, Version 1.0.
See accompanying file LICENSE_1_0.txt or copy at
http://www.boost.org/LICENSE_1_0.txt)

#############################################
*/

#pragma once
#include "boost/cstdint.hpp"
#include "boost/unordered_map.hpp"
#include "WorldObj.h"
#include "worldDefs.h"
#include <list>
#include <vector>
#include <omp.h>

using namespace boost;

namespace traffic_simulation
{
	struct nodeLock
	{
		ui64 guid; //car id
		driveDirection direction;
	};

	class Car;

	class Node : public WorldObj
	{
	public:
		Node(ui32 id, double x, double y) : WorldObj(x, y){_id = id;}
		void setID(ui32 id){_id = id;};
		ui32 getID()const {return _id;};
	private:
		ui32 _id;
		std::list<Car*> _locks;
	};

	class NodeHandler
	{
	public:
		static NodeHandler* getInstance();
		Node* getNode(ui32 id);
		static bool loaded(){return _bLoaded;};
		void addNodeLock(ui32 id, ui64 guid, driveDirection d);
		void removeNodeLock(ui32 id, ui64 guid);
		ui16 nodeLockCount(ui32 id, driveDirection d);
		bool isNodeLocked(ui32 id);
		worldPos* getMin(){return _min;};
		worldPos* getMax(){return _max;};
		~NodeHandler();
	private:

		typedef std::vector<std::list<nodeLock*>*> lockMap;
#ifdef _OPENMP
		std::vector<omp_lock_t*> _lock_mutex;
#endif
		NodeHandler();

		bool loadNodes();
		static NodeHandler* _instance;
		static bool _bInstance;
		static bool _bLoaded;
		worldPos* _min, *_max; //positions defining the largest and smallest coordinate pair.
		std::vector<Node*> _nodes;
		lockMap _locks; //aply own hash function here after node id's are normalized
	};
}