/*
Serial traffic simulation program for PaPo.

############################################

This application uses data from Open Street Map, henceforth
that data is made available under the Open Database License: http://opendatacommons.org/licenses/odbl/1.0/.
Any rights in individual contents ofthe database are licensed under 
the Database Contents License: http://opendatacommons.org/licenses/dbcl/1.0/
See more at: http://opendatacommons.org/licenses/odbl/#sthash.y0L9Yy1N.dpuf

############################################

Author: Martin Poppinga, Miriam von Heeremann, Marcel Ellermann
Date: 24.07.2013      
Distributed under the Boost Software License, Version 1.0.
See accompanying file LICENSE_1_0.txt or copy at
http://www.boost.org/LICENSE_1_0.txt)

#############################################
*/

#pragma once

#include "boost/cstdint.hpp"
#include "WorldObj.h"
#include "Street.h"
#include "worldDefs.h"
#include "NodeHandler.h"
#include "MapHandler.h"
#include <mpi.h>
#include "boost/numeric/ublas/matrix.hpp"
#include <list>

using namespace boost;

namespace traffic_simulation
{
	class Car;

	struct spawnContainer
	{
		ui64 id;
		SpawnerType type;
		ui32 size;
		std::list<std::pair<Node*, Street*>> spawnPoints;
	};

	class SpawnHandler
	{
	public:
		typedef std::vector<std::tuple<spawnContainer*,Node*,Street*>> spawnHolder;

		static SpawnHandler* getInstance();
		static bool loaded(){return _bLoaded;};

		void spawnCars();
		void notifyPathEndReached(Car* c);
		void registerForRemove(Car* c);
		void update();
		~SpawnHandler();
	private:
		SpawnHandler();
		void loadSpawns();
		ui64 _guidCounter;
		ui64 _sMaxSize[4];
		MPI_Op _uimaxOp;
		static SpawnHandler* _instance;
		static bool _bInstance;
		static bool _bLoaded;
		static int* _carPopMatrix;
		spawnHolder getFreeSpawns(const SpawnerType* types, const char arraySize);
		spawnHolder getSpawnPoints(ui32 t_Time);
		void getDestinationPoint(SpawnerType srcType, std::pair<Node*, Street*> &dPair);
		std::list<Car*> _delList;
		std::map<SpawnerType,std::vector<spawnContainer*>> _spawner;
	};
}