/*
Serial traffic simulation program for PaPo.

############################################

This application uses data from Open Street Map, henceforth
that data is made available under the Open Database License: http://opendatacommons.org/licenses/odbl/1.0/.
Any rights in individual contents ofthe database are licensed under 
the Database Contents License: http://opendatacommons.org/licenses/dbcl/1.0/
See more at: http://opendatacommons.org/licenses/odbl/#sthash.y0L9Yy1N.dpuf

############################################

Author: Martin Poppinga, Miriam von Heeremann, Marcel Ellermann
Date: 24.07.2013      
Distributed under the Boost Software License, Version 1.0.
See accompanying file LICENSE_1_0.txt or copy at
http://www.boost.org/LICENSE_1_0.txt)

#############################################
*/

#pragma once

#include "boost/cstdint.hpp"
#include "WorldObj.h"
#include "Street.h"
#include "worldDefs.h"
#include "NodeHandler.h"
#include "MapHandler.h"
#include <list>
#include <vector>

using namespace boost;

namespace traffic_simulation
{

	struct TrafficLight
	{
		std::list<Street*> regulatedStreets; //list of streets affected by traffig light state
		bool State; //0 = red; 1 = green;
		ui32 nodeID;
	};

	class TrafficLightSystem
	{
	public:
		void addTrafficLight(TrafficLight* tl);
		void update(ui8 t_step = 1);
		virtual void initSystemStates() = 0;
		virtual ~TrafficLightSystem();
	protected:
		std::vector<TrafficLight*> _lights;
		void setStreetLocks(TrafficLight* tl);
		ui16 _internalState;
		virtual void updateTrafficLightStates() = 0;
	};

	class SingleLightSystem : virtual public TrafficLightSystem
	{
	public:
		void initSystemStates();
	protected:
		void updateTrafficLightStates();
		ui8 phase;
	};

	class ThreeLightSystem : virtual public TrafficLightSystem
	{
	public:
		void initSystemStates();
	protected:
		void updateTrafficLightStates();
		ui8 phase;
	};

	class FourLightSystem : virtual public TrafficLightSystem
	{
	public:
		void initSystemStates();
	protected:
		void updateTrafficLightStates();
		ui8 phase;
	};

	class TrafficLightSystemFactory
	{
	public:
		static TrafficLightSystem* createTrafficLightSystem(ui8 tlCount);
	};

	class TrafficLightHandler
	{
	public:
		static TrafficLightHandler* getInstance();
		static bool loaded(){return _bLoaded;};

		void notifyPathEndReached(Car* c);
		void update(ui8 t_step = 1);
		~TrafficLightHandler();
	private:
		TrafficLightHandler();
		void loadTrafficLights();
		std::list<TrafficLightSystem*> _tlSystems;
		static TrafficLightHandler* _instance;
		static bool _bInstance;
		static bool _bLoaded;
	};
}