/*
Serial traffic simulation program for PaPo.

############################################

This application uses data from Open Street Map, henceforth
that data is made available under the Open Database License: http://opendatacommons.org/licenses/odbl/1.0/.
Any rights in individual contents ofthe database are licensed under 
the Database Contents License: http://opendatacommons.org/licenses/dbcl/1.0/
See more at: http://opendatacommons.org/licenses/odbl/#sthash.y0L9Yy1N.dpuf

############################################

Author: Martin Poppinga, Miriam von Heeremann, Marcel Ellermann
Date: 24.07.2013      
Distributed under the Boost Software License, Version 1.0.
See accompanying file LICENSE_1_0.txt or copy at
http://www.boost.org/LICENSE_1_0.txt)

#############################################
*/
#define _USE_MATH_DEFINES
#include "WorldObj.h"
#include <math.h>

namespace traffic_simulation
{
	WorldObj::WorldObj(double x, double y)
	{
		_pos = new worldPos(x, y);
	}

	WorldObj::WorldObj(worldPos* pos)
	{
		_pos = new worldPos(pos->getX(), pos->getY());
	}

	WorldObj::~WorldObj()
	{
		delete _pos;
	}

	worldPos* WorldObj::getPos() const
	{
		return _pos;
	}

#define degrees_to_radians M_PI/180.0
#define EARTH_RADIUS_METER 6356752

	double worldPos::distance( worldPos *pos) const
	{
		if(pos->_x == _x && pos->_y == _y)
			return 0;
		/*Convert latitude and longitude to
		  spherical coordinates in radians.*/
		//phi = 90 - latitude
		double phi1 = (90.0 - _x)*degrees_to_radians;
		double phi2 = (90.0 - pos->getX())*degrees_to_radians;

		// theta = longitude
		double theta1 = _y * degrees_to_radians;
		double theta2 = pos->getY() * degrees_to_radians;
		/*
			Compute spherical distance from spherical coordinates.
			For two locations in spherical coordinates
		    (1, theta, phi) and (1, theta, phi)
			cosine( arc length ) = sin phi sin phi' cos(theta-theta') + cos phi cos phi'
			distance = rho * arc length
		*/

		double _c = sin(phi1)*sin(phi2)*cos(theta1 - theta2) + cos(phi1)*cos(phi2);
		double _arc = acos(_c);
		return _arc * EARTH_RADIUS_METER;
	}
}