/*
	Serial traffic simulation program for PaPo.

	############################################

	This application uses data from Open Street Map, henceforth
	that data is made available under the Open Database License: http://opendatacommons.org/licenses/odbl/1.0/.
	Any rights in individual contents ofthe database are licensed under 
	the Database Contents License: http://opendatacommons.org/licenses/dbcl/1.0/
	See more at: http://opendatacommons.org/licenses/odbl/#sthash.y0L9Yy1N.dpuf

	############################################

	Author: Martin Poppinga, Miriam von Heeremann, Marcel Ellermann
	Date: 24.07.2013      
	Distributed under the Boost Software License, Version 1.0.
	    See accompanying file LICENSE_1_0.txt or copy at
	          http://www.boost.org/LICENSE_1_0.txt)

	#############################################
*/

#pragma once
#include "boost/cstdint.hpp"
#include "worldDefs.h"


namespace traffic_simulation
{

	/// <summary>
	/// Class to represent the position of any object within the world
	/// </summary>
	class worldPos
	{
	public:
		worldPos(double x, double y)
		{
			_x = x;
			_y = y;
		}
		void setX(double x){x = x;};
		void setY(double y){y = y;};
		void setPos(double x, double y){_x = x; _y=y;};

		double getX() const{return _x;};
		double getY() const{return _y;};

		double distance( worldPos *pos) const;

	private:
		double _x;
		double _y;
	};

	class WorldObj
	{
	public:
		WorldObj(double x, double y);
		WorldObj(worldPos* pos);
		~WorldObj();

		void update(uint8_t step = 1);

		worldPos* getPos() const;

	private:
		worldPos* _pos;
	};
}