/*
Serial traffic simulation program for PaPo.

############################################

This application uses data from Open Street Map, henceforth
that data is made available under the Open Database License: http://opendatacommons.org/licenses/odbl/1.0/.
Any rights in individual contents ofthe database are licensed under 
the Database Contents License: http://opendatacommons.org/licenses/dbcl/1.0/
See more at: http://opendatacommons.org/licenses/odbl/#sthash.y0L9Yy1N.dpuf

############################################

Author: Martin Poppinga, Miriam von Heeremann, Marcel Ellermann
Date: 24.07.2013      
Distributed under the Boost Software License, Version 1.0.
See accompanying file LICENSE_1_0.txt or copy at
http://www.boost.org/LICENSE_1_0.txt)

#############################################
*/

#pragma once
#include "boost/cstdint.hpp"
#include "world.h"
#include "boost/iostreams/stream.hpp"
#include <ostream>
#include <omp.h>

//debug definitions
#define TF_DEBUG_LEVEL 1	//3=FULL_DETAIL_DEBUG 2 = DETAIL_DEBUG, 1=MAIN_DEBUG, 0=NO_DEBUG
#if(TF_DEBUG_LEVEL == 3)
#define TF_DEBUG_DETAIL_FULL
#define TF_DEBUG_DETAIL
#define TF_DEBUG_MAIN
#elif(TF_DEBUG_LEVEL == 2)
#define TF_DEBUG_DETAIL
#define TF_DEBUG_MAIN
#elif(TF_DEBUG_LEVEL==1)
#define TF_DEBUG_MAIN
#endif

//#define LOG_PRINT_CAR_LOG //output car log to console
#define LOG_PRINT_WORLD_LOG //output world log to console
//#define LOG_PRINT_STREET_LOG //ouput street log to console
//#define LOG_PRINT_TRAFFIC_LOG //ouput traffic light log to console

#define CAR_DUMP
#define CAR_DUMP_STEP 10


#define MAP_DATA_VERSION "V0.6.6.0"
#define CHECK_VERSION(A) (strcmp(A,MAP_DATA_VERSION)==0)
//singletion definitions
#define LOG traffic_simulation::Log::getInstance()
#define MAP traffic_simulation::MapHandler::getInstance()
#define SPAWNS traffic_simulation::SpawnHandler::getInstance()
#define NODES  traffic_simulation::NodeHandler::getInstance()
#define MSG MessageHandler::getInstance()
#define TRAFFIC_LIGHTS traffic_simulation::TrafficLightHandler::getInstance()
#define CONFIG traffic_simulation::ConfigHandler::getInstance()

#define WORLD_TIME_STAMP traffic_simulation::world::getTimeStamp()
#define FILE_NODES 0
#define FILE_STREETS 1
#define FILE_INTERSECTIONS 2
#define FILE_SPAWNS 3
#define FILE_CROSSROADS 4
#ifdef _WIN32
#define PATH_CAR_DUMP CONFIG->getPath("PATH_CAR_DUMP_WIN32")
#define PATH_DATA  CONFIG->getPath("PATH_DATA_WIN32")
#else
#define PATH_CAR_DUMP CONFIG->getPath("PATH_CAR_DUMP_UNIX")
#define PATH_DATA  CONFIG->getPath("PATH_DATA_UNIX")
#endif
#define LOGIC_TIME_TO_SEC CONFIG->getDouble("LOGIC_TIME_TO_SEC") //1/N Logic Time stamp are 1 second
#define SEC_TO_LOGIC_TIME CONFIG->getDouble("SEC_TO_LOGIC_TIME")
#define CAR_LENGTH CONFIG->getDouble("CAR_LENGTH") //4m average length
#define MAX_CARS CONFIG->getUInt32("MAX_CARS")//max number of cars that are driving on the map4
#define SIMULATION_TICKS CONFIG->getUInt32("SIMULATION_TICKS") //updates that are executed 3600 currently = 1h
#define SPAWN_LIMIT_PER_TICK CONFIG->getUInt32("SPAWN_LIMIT_PER_TICK")
#define INPUT_MAP CONFIG->getPath("INPUT_MAP")

//funtions

#define _FMT_TO_C_STR_(A) boost::str(A).c_str()

typedef boost::uint8_t  ui8;
typedef boost::uint16_t ui16;
typedef boost::uint32_t ui32;
typedef boost::uint64_t ui64;

///<summary>
/// defines the direction the car passes the nodes of a street.
/// forward if it's in the direction of the standard iterator of the streets' node list
/// or backward if's moving in the direction of the reverse iterator of the street's node list
///</summary>
enum driveDirection
{
	forward, backward, both
};

enum SpawnerType{
	residential, retail, industrial, commercial
};

