import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.ArrayList;
import java.util.List;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;

/**
 * Verwandelt die Ausgabe des c++-Programms in eine Datei, in der die
 * Koordinaten der Autos pro Zeitschritt aufgelistet sind.
 * 
 * @author Miriam von Heereman, Marcel Ellermann, Martin Poppinga
 * 
 */
public class GrafischeDarstellung
{
	static private ArrayList<Node> _nodes = new ArrayList<Node>();
	static private ArrayList<Street> _streets = new ArrayList<Street>();
	static private String _pfad;
	static private String _ort;

	public static void main(String[] args) throws IOException
	{
		main(args[0]);
	}

	/**
	 * Startet das Programm, ldt die Ausgabe-Datei ein und liest die Bytes.
	 * Diese werden zu StraenId und Abstand vom Startknoten umgewandelt und
	 * anschlieend in eine Liste gespeichert. Daraus sollen dann die
	 * Koordinaten errechnet werden, an denen ein Punkt fr ein Auto gezeichnet
	 * wird.
	 * 
	 * @param args
	 * @throws IOException
	 */
	public static void main(String ort) throws IOException
	{
		// Der Pfad an dem das Jar-File liegt.
		String pfad = GrafischeDarstellung.class.getProtectionDomain()
				.getCodeSource().getLocation().getPath();
		_pfad = pfad.replaceAll("Koordinaten.jar", "");
		_ort = ort;
		System.out.println(GrafischeDarstellung.class.getProtectionDomain()
				.getCodeSource().getLocation().getPath());
		FileInputStream fin = new FileInputStream(_pfad
				+ "Traffic_Simulation_Seriell/car_dump.dat");		
		List<Double> meter = ausleser(fin);
		readNodes();
		readStreets();
		File ausgabe = new File(_pfad + "Karten/input/ausgabe_" + _ort + ".dat");
		FileWriter writer = new FileWriter(ausgabe, true);
		int zahl = meter.size();
		for (int i = 0; i < zahl;)
		{
			double anzahl2 = (meter.get(i));
			i = i + 1;
			int anzahl = (int) anzahl2;
			writer.write(anzahl + "\r\n");
			// Schleife handelt einen Zeitschrittab.
			for (int j = 0; j < anzahl; j++)
			{
				double strid2 = meter.get(i); // straenid
				Street s = streetReader((long) strid2);
				double autolaenge = meter.get(i + 1); // die lnge die sich das
														// auto
														// auf der gesamten
														// Strae
														// bewegt hat.
				// Ruft mit der Straenid und der Lnge ab, zwischen welchen 2
				// Knotenpunkten der Strae sich das Auto befindet. In nodes
				// werden die KnotenID und die Lngen der beiden Knoten jeweils
				// als Double zurckgegeben. Die Variablen mit "2" sind der
				// jeweils grere knoten.
				Node klein = s.kleinerKnoten(autolaenge);
				Node gro = s.groerKnoten(autolaenge);
				long kleinId = klein._id;
				long groId = gro._id;
				Node kleinkoord = knotenleser(kleinId);
				Node grokoord = knotenleser(groId);
				// autolaenge: autolaenge die das Auto auf der Strae unterwegs
				// ist.
				// kleinX: 1. Koordinate des 1. Knotens
				// kleinY: 2. Koordinate des 1. Knotens
				// kleinlnge: abstand des 1. knotens zum 0 knoten.
				// Fr 2. Knoten statt klein, gro.
				double kleinx = kleinkoord._x;
				double kleiny = kleinkoord._y;
				double kleinlaenge = kleinkoord._laenge;
				double grox = grokoord._x;
				double groy = grokoord._y;
				double grolaenge = grokoord._laenge;
				double knotenabstand = grolaenge - kleinlaenge;
				double autolnge = autolaenge - kleinlaenge;
				double fortschrittprozent = (autolnge / knotenabstand);
				double autokoordinatex = (((grox - kleinx) * fortschrittprozent) + kleinx);
				double autokoordinatey = (((groy - kleiny) * fortschrittprozent) + kleiny);
				if (String.valueOf(autokoordinatey).equals("NaN")
						|| String.valueOf(autokoordinatey).equals("NaN"))
				{

				} else
				{
					writer.write(String.valueOf(autokoordinatex) + ";");
					writer.write(String.valueOf(autokoordinatey));
					writer.write("\r\n");
				}
				i = i + 2;
			}
		}
		fin.close();
		writer.close();
	}

	private static void readNodes() throws FileNotFoundException
	{
		FileReader f = new FileReader(_pfad + "Karten/output/" + _ort
				+ "/nodes_" + _ort + "_tmp.dat");
		BufferedReader br = new BufferedReader(f);
		String s;
		try
		{
			s = br.readLine();
			s = br.readLine();
			s = br.readLine();
			while ((s = br.readLine()) != null)
			{
				String[] nodes = s.split(";");
				Node d = new Node(Long.parseLong(nodes[0]),
						Double.parseDouble(nodes[1]),
						Double.parseDouble(nodes[2]));
				_nodes.add(d);
			}
			f.close();
			// Liste mit Zeilenweise der knoten-koordinaten datei.
		} catch (IOException e)
		{

			e.printStackTrace();
		}
	}

	/**
	 * Liest den Knoten mit der knotenId ein.
	 * 
	 * @param knotenId
	 * @return
	 * @throws FileNotFoundException
	 */
	public static Node knotenleser(long knotenId)
	{
		int i = 1;
		Node n = _nodes.get(0);
		while (n.idAbfrage(knotenId) != true)
		{
			n = _nodes.get(i);
			i++;
		}
		return n;
	}

	/**
	 * Liest aus dem InputStream die Bytes aus und wandelt sie in die
	 * StraenId's und die Lnge der Fortbewegung des Autos um.
	 * 
	 * @param fin
	 * @return
	 * @throws IOException
	 */
	public static List<Double> ausleser(FileInputStream fin) throws IOException
	{
		BufferedInputStream bufin = new BufferedInputStream(fin);
		BufferedInputStream bufin2 = new BufferedInputStream(fin);
		int lnge = bufin2.available();
		List<Double> meter = new ArrayList<Double>();
		// List strae = new ArrayList();
		for (int j = 0; j < lnge;)
		{
			byte[] b = new byte[4];
			bufin.read(b);
			long anzahllong = byteArrayToInt(b);
			int anzahl = (int) anzahllong;
			meter.add((double) anzahl);
			j = j + 4 + (anzahl * 12);
			for (int i = 0; i < anzahl; i++)
			{
				byte[] a = new byte[4];
				bufin.read(a);
				meter.add((double) byteArrayToInt(a));
				// System.out.println(straeId);
				byte[] c = new byte[8];
				bufin.read(c);
				meter.add(toDouble(c));
				// System.out.println(achter);
			}
		}
		return meter;
	}

	/**
	 * Legt eine Liste von allen Straen an, die die Straenid mit all Ihren
	 * Knoten, deren Id's und Entfernung zum Startpunkt enthlt.
	 * 
	 * @throws FileNotFoundException
	 */

	private static void readStreets() throws FileNotFoundException
	{
		FileReader f = new FileReader(_pfad + "Karten/output/" + _ort
				+ "/fullways_" + _ort + "_tmp.xml");
		SAXBuilder sxb = new SAXBuilder();
		Document document;
		try
		{
			document = sxb.build(f);
			// Wurzelelement
			Element element = document.getRootElement();
			// Alle Wege
			List<Element> list = element.getChildren();
			for (int i = 0; i < list.size(); i++)
			{
				Element weg = list.get(i);
				ArrayList<Node> nodes = new ArrayList<Node>();
				long id = Long.parseLong(weg.getChild("id").getText());
				List<Element> knoten = weg.getChild("Nodes").getChildren();
				for (int j = 0; j < knoten.size(); j++)
				{
					Element node = knoten.get(j);
					Node n = new Node(Long.parseLong(node
							.getAttributeValue("id")), Double.parseDouble(node
							.getText()));
					nodes.add(n);
				}
				Street street = new Street(id, nodes);
				_streets.add(street);
			}

		} catch (JDOMException e)
		{
			e.printStackTrace();
		} catch (IOException e)
		{
			e.printStackTrace();
		}
	}

	/**
	 * Sucht die Strae mit der StraenId strid aus der Straenliste heraus.
	 * 
	 * @param strid
	 * @return
	 */

	public static Street streetReader(long strid)
	{
		int i = 1;
		Street s = _streets.get(0);
		while (s.idAbfrage(strid) != true)
		{
			s = _streets.get(i);
			i++;
		}
		return s;
	}

	/**
	 * Wandelt ein Byte-Array (Lnge 4 Bytes) in einen Long-Wert um.
	 * 
	 * @param b
	 * @return
	 */
	public static long byteArrayToInt(byte[] b)
	{
		long value = 0;
		for (int i = 0; i < b.length; i++)
		{
			value += ((long) b[i] & 0xffL) << (8 * i);
		}
		return value;
	}

	/**
	 * Wandelt ein Byte-Array (Lnge 8 Bytes) in einen Double-Wert um.
	 * 
	 * @param bytes
	 * @return
	 */
	public static double toDouble(byte[] bytes)
	{
		double d = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN)
				.getDouble();
		return d;
	}
}
